# Generated by roxygen2: do not edit by hand

export(SPAS.autopool)
export(SPAS.fit.model)
export(SPAS.print.model)
import(MASS)
import(TMB)
import(msm)
import(plyr)
import(stats)
importFrom(Matrix,bdiag)
importFrom(checkmate,assert_logical)
importFrom(reshape2,melt)
importFrom(utils,packageVersion)
useDynLib(SPAS)
