% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/roxygentags.R
\name{dummy}
\alias{dummy}
\title{Roxygen commands}
\usage{
dummy()
}
\description{
Roxygen commands
}
\keyword{internal}
