% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{.onAttach}
\alias{.onAttach}
\title{Message to display when package is loaded}
\usage{
.onAttach(libname, pkgname)
}
\description{
Message to display when package is loaded
}
\keyword{internal}
